//
//  AudioPlayer.swift
//  pinniped-ios
//
//  Created by Nathan Mattes on 24.08.23.
//

import Foundation
import AVFoundation

class AudioPlayer {

  static let shared = AudioPlayer()

  private let audioPlayer: AVQueuePlayer
  private var playerItems: [AVPlayerItem]
  private let fileManager: FileManager

  init(fileManager: FileManager = .default) {
    self.fileManager = fileManager
    self.audioPlayer = AVQueuePlayer()
    self.audioPlayer.volume = 1.0
    self.audioPlayer.actionAtItemEnd = .advance
    self.playerItems = []
  }

  func play(track: Track, from album: Album, trackList: [Track]) {

    do {
      let url = try FileManager.default.url(for: track)
      try AVAudioSession.sharedInstance().setActive(true)
      let playerItem = AVPlayerItem(url: url)

      audioPlayer.removeAllItems()
      audioPlayer.replaceCurrentItem(with: playerItem)
      //TODO: Insert trackList.playerItems
      //TODO: Logik, um weitere Tracks herunterzuladen. DataSource? Downloader
//      let tracksAfter = trackList
//        .filter { $0.position > track.position }
//        .compactMap {  }

      audioPlayer.play()

      NotificationCenter.default.post(name: .audioPlaybackStarted, object: nil)

      print("Playing \(track.title) from \(album.title)")
    } catch {
      print(error)
    }
  }

  func pause() {
    audioPlayer.pause()
  }

  func play() {
    audioPlayer.play()
  }

  func nextTrack() {
    //TODO: Implement, see: https://stackoverflow.com/questions/34971839/better-way-to-do-skip-to-previous-with-avqueueplayer
  }

  func previousTrack() {
    //TODO: Implement, see: https://stackoverflow.com/questions/34971839/better-way-to-do-skip-to-previous-with-avqueueplayer
  }

  var isPlaying: Bool {
    return audioPlayer.rate != 0 && audioPlayer.error == nil
  }
}
