//
//  Endpoints.swift
//  pinniped-ios
//
//  Created by Nathan Mattes on 08.08.23.
//

import Foundation

enum FunkwhaleEndpoints {

  case registerApp
  case authorize
  case getToken

  case artists
  case albums
  case tracks
  case listenToTack(Track)

  func url(baseURL: URL) -> URL? {
    switch self {
      case .registerApp:
        return baseURL.appending(path: "/api/v1/oauth/apps")
      case .authorize:
        return baseURL.appendingPathComponent("/authorize/")
      case .getToken:
        return baseURL.appendingPathComponent("/api/v1/oauth/token/")

      case .artists:
        return baseURL.appendingPathComponent("/api/v1/artists/")
      case .albums:
        return baseURL.appendingPathComponent("/api/v1/albums/")
      case .tracks:
        return baseURL.appendingPathComponent("/api/v1/tracks/")
      case .listenToTack(let track):
        guard let listenURL = track.listen_url else { return nil }

        return baseURL.appendingPathComponent(listenURL).appending(queryItems: [.init(name: "to", value: "mp3")])
    }
  }

  var method: HTTPMethod {
    switch self {
      case .registerApp:
        return .POST
      case .authorize:
        return .GET
      case .getToken:
        return .POST
      case .artists:
        return .GET
      case .albums:
        return .GET
      case .tracks:
        return .GET
      case .listenToTack(_):
        return .GET
    }
  }

  func request(baseURL: URL, body: Data? = nil, token: String? = nil, username: String? = nil, password: String? = nil) -> URLRequest? {

    guard let url = self.url(baseURL: baseURL) else { return nil }
    
    var request = URLRequest(url: url)
    request.httpBody = body
    request.httpMethod = method.rawValue

    if let token = token {
      request.addValue("Bearer \(token)", forHTTPHeaderField: "Authorization")
    } else if let username = username, let password = password,
              let basicAuth = "\(username):\(password)".data(using: .utf8) {
      request.addValue("Basic \(basicAuth.base64EncodedString())", forHTTPHeaderField: "Authorization")
    }

    switch self {
      case .registerApp:
        request.addValue("application/json", forHTTPHeaderField: "accept")
        request.addValue("application/json", forHTTPHeaderField: "Content-Type")
      case .authorize:
        break
      case .getToken:
        request.addValue("application/x-www-form-urlencoded", forHTTPHeaderField: "ContentType")
      case .artists:
        break
      case .albums:
        break
      case .tracks:
        break
      case .listenToTack(_):
        break
    }

    return request
  }
}
