//
//  AlbumDetailsCoverTableViewCell.swift
//  pinniped-ios
//
//  Created by Nathan Mattes on 10.08.23.
//

import UIKit
import Kingfisher

class AlbumDetailsCoverTableViewCell: UITableViewCell {
  static let reuseIdentifier = "AlbumDetailsCoverTableViewCell"

  let coverImageView: UIImageView
  private let contentStackView: UIStackView

  override init(style: UITableViewCell.CellStyle, reuseIdentifier: String?) {

    coverImageView = UIImageView()
    coverImageView.contentMode = .scaleAspectFill
    coverImageView.layer.cornerRadius = 8.0
    coverImageView.layer.maskedCorners = [.layerMaxXMaxYCorner, .layerMaxXMinYCorner, .layerMinXMaxYCorner, .layerMinXMinYCorner]
    coverImageView.layer.masksToBounds = true
    coverImageView.translatesAutoresizingMaskIntoConstraints = false

    contentStackView = UIStackView(arrangedSubviews: [coverImageView])
    contentStackView.translatesAutoresizingMaskIntoConstraints = false
    contentStackView.axis = .vertical
    contentStackView.alignment = .center

    super.init(style: style, reuseIdentifier: reuseIdentifier)

    selectionStyle = .none

    backgroundColor = .systemGroupedBackground
    contentView.addSubview(contentStackView)
    setupConstraints()
  }

  required init?(coder: NSCoder) { fatalError("init(coder:) has not been implemented") }

  private func setupConstraints() {
    let constraints = [
      contentStackView.topAnchor.constraint(equalTo: contentView.topAnchor),
      contentStackView.leadingAnchor.constraint(equalTo: contentView.leadingAnchor),
      contentView.trailingAnchor.constraint(equalTo: contentStackView.trailingAnchor),
      contentView.bottomAnchor.constraint(greaterThanOrEqualTo: contentStackView.bottomAnchor),

      coverImageView.heightAnchor.constraint(equalTo: coverImageView.widthAnchor),
      coverImageView.widthAnchor.constraint(lessThanOrEqualToConstant: 200),
    ]

    NSLayoutConstraint.activate(constraints)
  }

  func configure(with album: Album) {
    if let imageURL = album.cover?.urls.large_square_crop {
      coverImageView.kf.setImage(with: URL(string: imageURL), placeholder: Asset.albumPlaceholder.image)
    } else {
      coverImageView.image = Asset.albumPlaceholder.image
    }
  }
}
