//
//  AlbumsCollectionViewCell.swift
//  pinniped-ios
//
//  Created by Nathan Mattes on 10.08.23.
//

import UIKit
import Kingfisher

class AlbumsCollectionViewCell: UICollectionViewCell {
  static let reuseIdentifier = "AlbumsCollectionViewCell"

  let coverImageView: UIImageView
  let titleLabel: UILabel
  let secondaryLabel: UILabel

  private let contentStackView: UIStackView

  override init(frame: CGRect) {
    coverImageView = UIImageView()
    coverImageView.contentMode = .scaleAspectFill
    coverImageView.layer.cornerRadius = 8.0
    coverImageView.layer.maskedCorners = [.layerMaxXMaxYCorner, .layerMaxXMinYCorner, .layerMinXMaxYCorner, .layerMinXMinYCorner]
    coverImageView.layer.masksToBounds = true

    titleLabel = UILabel()
    titleLabel.font = UIFont.preferredFont(forTextStyle: .headline)
    
    secondaryLabel = UILabel()
    secondaryLabel.font = UIFont.preferredFont(forTextStyle: .subheadline)

    contentStackView = UIStackView(arrangedSubviews: [coverImageView, titleLabel, secondaryLabel])
    contentStackView.translatesAutoresizingMaskIntoConstraints = false
    contentStackView.axis = .vertical
    contentStackView.alignment = .leading
    contentStackView.setCustomSpacing(8, after: coverImageView)
    contentStackView.setCustomSpacing(4, after: titleLabel)

    super.init(frame: frame)

    contentView.addSubview(contentStackView)
    setupConstraints()
  }

  required init?(coder: NSCoder) { fatalError("init(coder:) has not been implemented") }

  override func prepareForReuse() {
    super.prepareForReuse()

    coverImageView.image = nil
  }

  private func setupConstraints() {
    let constraints = [
      contentStackView.topAnchor.constraint(equalTo: contentView.topAnchor),
      contentStackView.leadingAnchor.constraint(equalTo: contentView.leadingAnchor),
      contentView.trailingAnchor.constraint(equalTo: contentStackView.trailingAnchor),
      contentView.bottomAnchor.constraint(greaterThanOrEqualTo: contentStackView.bottomAnchor),

      coverImageView.heightAnchor.constraint(equalTo: coverImageView.widthAnchor),
    ]

    NSLayoutConstraint.activate(constraints)
  }

  func update(with album: Album) {
    //TODO: Implement
    if let imageURL = album.cover?.urls.large_square_crop {
      coverImageView.kf.setImage(with: URL(string: imageURL))
    } else {
      coverImageView.image = Asset.albumPlaceholder.image
    }
    titleLabel.text = album.title
    if let releaseDate = album.release_date, let year = PinnipedDateFormatter.shared.getYear(fromAlbumReleaseDateString: releaseDate) {
      secondaryLabel.text = "\(year)"
    } else {
      secondaryLabel.text = "-"
    }
  }
}
