//
//  AlbumsViewController.swift
//  pinniped-ios
//
//  Created by Nathan Mattes on 10.08.23.
//

import UIKit

enum AlbumsSection: Hashable {
  case main
}

enum AlbumItem: Hashable {
  case album(Album)
}

protocol AlbumsViewControllerDelegate: AnyObject {
  func showAccountSettings(_ viewController: AlbumsViewController)
  func reloadAlbums(_ viewController: AlbumsViewController)
  func showAlbumDetails(_ viewController: UIViewController, album: Album)
}

class AlbumsViewController: UIViewController {

  weak var delegate: AlbumsViewControllerDelegate?
  var dataSource: UICollectionViewDiffableDataSource<AlbumsSection, AlbumItem>?

  var contentView: AlbumsView {
    view as! AlbumsView
  }

  var albums: [Album] = []

  init() {
    super.init(nibName: nil, bundle: nil)

    title = L10n.Albums.title
    navigationItem.rightBarButtonItem = UIBarButtonItem(image: UIImage(systemName: "person.circle"), style: .done, target: self, action: #selector(AlbumsViewController.showAccountSettings(_:)))
    NotificationCenter.default.addObserver(self, selector: #selector(audioPlaybackStarted(_:)), name: .audioPlaybackStarted, object: nil)
  }

  required init?(coder: NSCoder) { fatalError("init(coder:) has not been implemented") }

  override func loadView() {
    let albumsView = AlbumsView(frame: .zero)
    albumsView.collectionView.refreshControl?.addTarget(self, action: #selector(AlbumsViewController.reloadAlbums(_:)), for: .valueChanged)

    let dataSource = UICollectionViewDiffableDataSource<AlbumsSection, AlbumItem>(collectionView: albumsView.collectionView) { collectionView, indexPath, itemIdentifier in

      guard let cell = collectionView.dequeueReusableCell(withReuseIdentifier: AlbumsCollectionViewCell.reuseIdentifier, for: indexPath) as? AlbumsCollectionViewCell else { fatalError("Wrong cell WTF") }

      switch itemIdentifier {
        case .album(let album):
          cell.update(with: album)
      }

      return cell
    }

    self.dataSource = dataSource
    albumsView.collectionView.delegate = self

    view = albumsView
  }

  override func viewDidLoad() {
    super.viewDidLoad()
    delegate?.reloadAlbums(self)
  }

  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)

    if AudioPlayer.shared.isPlaying {
      contentView.collectionView.contentInset.bottom = CGFloat.scrollViewBottomInsetForWidget
    }
  }

  //MARK: - Notitifications

  @objc
  func audioPlaybackStarted(_ notification: Notification) {
    if AudioPlayer.shared.isPlaying {
      contentView.collectionView.contentInset.bottom = CGFloat.scrollViewBottomInsetForWidget
    }
  }

  //MARK: - Actions

  @objc
  func showAccountSettings(_ sender: Any) {
    delegate?.showAccountSettings(self)
  }

  @objc
  func reloadAlbums(_ sender: Any) {
    delegate?.reloadAlbums(self)
  }


  func update(with albums: [Album], animated: Bool) {

    self.albums = albums

    var snapshot = NSDiffableDataSourceSnapshot<AlbumsSection, AlbumItem>()
    snapshot.appendSections([.main])
    snapshot.appendItems(albums.compactMap { .album($0) })

    dataSource?.apply(snapshot, animatingDifferences: animated)
  }
}

extension AlbumsViewController: UICollectionViewDelegate {
  func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
    let album = albums[indexPath.item]

    delegate?.showAlbumDetails(self, album: album)
  }
}
