//
//  ArtistAlbumsViewController.swift
//  pinniped-ios
//
//  Created by Nathan Mattes on 10.08.23.
//

import UIKit

enum ArtistAlbumsSection: Hashable {
  case main
}

enum ArtistAlbumItem: Hashable {
  case album(Album)
}

protocol ArtistAlbumsViewControllerDelegate: AnyObject {
  func showAlbumDetails(_ viewController: UIViewController, album: Album)
}

class ArtistAlbumsViewController: UIViewController {

  weak var delegate: ArtistAlbumsViewControllerDelegate?
  var dataSource: UICollectionViewDiffableDataSource<ArtistAlbumsSection, ArtistAlbumItem>?

  var contentView: ArtistAlbumsView {
    view as! ArtistAlbumsView
  }

  var albums: [Album]

  init(albums: [Album], artist: Artist) {

    self.albums = albums

    super.init(nibName: nil, bundle: nil)

    title = artist.name
    NotificationCenter.default.addObserver(self, selector: #selector(audioPlaybackStarted(_:)), name: .audioPlaybackStarted, object: nil)
  }

  required init?(coder: NSCoder) { fatalError("init(coder:) has not been implemented") }

  override func loadView() {
    let albumsView = ArtistAlbumsView(frame: .zero)

    let dataSource = UICollectionViewDiffableDataSource<ArtistAlbumsSection, ArtistAlbumItem>(collectionView: albumsView.collectionView) { collectionView, indexPath, itemIdentifier in

      guard let cell = collectionView.dequeueReusableCell(withReuseIdentifier: AlbumsCollectionViewCell.reuseIdentifier, for: indexPath) as? AlbumsCollectionViewCell else { fatalError("Wrong cell WTF") }

      switch itemIdentifier {
        case .album(let album):
          cell.update(with: album)
      }

      return cell
    }

    self.dataSource = dataSource
    albumsView.collectionView.delegate = self

    view = albumsView
  }

  override func viewDidLoad() {
    super.viewDidLoad()

    update(with: albums, animated: true)
  }

  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)

    if AudioPlayer.shared.isPlaying {
      contentView.collectionView.contentInset.bottom = CGFloat.scrollViewBottomInsetForWidget
    }
  }

  //MARK: - Notitifications

  @objc
  func audioPlaybackStarted(_ notification: Notification) {
    if AudioPlayer.shared.isPlaying {
      contentView.collectionView.contentInset.bottom = CGFloat.scrollViewBottomInsetForWidget
    }
  }

  //MARK: - Actions

  func update(with albums: [Album], animated: Bool) {

    self.albums = albums

    var snapshot = NSDiffableDataSourceSnapshot<ArtistAlbumsSection, ArtistAlbumItem>()
    snapshot.appendSections([.main])
    snapshot.appendItems(albums.compactMap { .album($0) })

    dataSource?.apply(snapshot, animatingDifferences: animated)
  }
}

extension ArtistAlbumsViewController: UICollectionViewDelegate {
  func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {

    let album = albums[indexPath.item]

    delegate?.showAlbumDetails(self, album: album)
  }
}
