//
//  ConnectToFunkwhaleViewController.swift
//  pinniped-ios
//
//  Created by Nathan Mattes on 08.08.23.
//

import UIKit


enum ConnectToFunkwhaleSection: Hashable {
  case main
}

enum ConnectToFunkwhaleEntry: Hashable {
  case url
}

protocol ConnectToFunkwhaleViewControllerDelegate: AnyObject {
  func login(_ viewController: ConnectToFunkwhaleViewController, urlString: String)
}

class ConnectToFunkwhaleViewController: UIViewController {

  weak var delegate: ConnectToFunkwhaleViewControllerDelegate?
  var dataSource: UITableViewDiffableDataSource<ConnectToFunkwhaleSection, ConnectToFunkwhaleEntry>?

  var contentView: ConnectToFunkwhaleView {
    view as! ConnectToFunkwhaleView
  }

  var urlString: String?

  init() {
    super.init(nibName: nil, bundle: nil)

    title = L10n.Welcome.ConnectServer.title
    view.backgroundColor = .systemGroupedBackground

    navigationItem.rightBarButtonItem = UIBarButtonItem(title: "Login", style: .done, target: self, action: #selector(login(_:)))
    navigationItem.rightBarButtonItem?.isEnabled = false
  }

  required init?(coder: NSCoder) { fatalError("init(coder:) has not been implemented") }

  override func loadView() {
    let contentView = ConnectToFunkwhaleView(frame: .zero)

    let dataSource = UITableViewDiffableDataSource<ConnectToFunkwhaleSection, ConnectToFunkwhaleEntry>(tableView: contentView.tableView) { tableView, indexPath, itemIdentifier in
      guard let cell = tableView.dequeueReusableCell(withIdentifier: ConnectToFunkwhaleTableViewCell.reuseIdentifier, for: indexPath) as? ConnectToFunkwhaleTableViewCell else { fatalError("WTF wrong Cell") }

      cell.textField.addTarget(self, action: #selector(ConnectToFunkwhaleViewController.textFieldDidChangeText(_:)), for: .editingChanged)

      return cell
    }

    self.dataSource = dataSource
    contentView.tableView.delegate = self
    view = contentView
  }

  override func viewDidLoad() {
    super.viewDidLoad()

    var snapshot = NSDiffableDataSourceSnapshot<ConnectToFunkwhaleSection, ConnectToFunkwhaleEntry>()
    snapshot.appendSections([.main])
    snapshot.appendItems([.url], toSection: .main)
    dataSource?.apply(snapshot)
  }

  //MARK: - Actions

  @objc
  func login(_ sender: Any) {

    guard let urlString else { return }

    delegate?.login(self, urlString: urlString)
  }

  @objc
  func textFieldDidChangeText(_ textField: UITextField) {
    urlString = textField.text

    if urlString == nil {
      navigationItem.rightBarButtonItem?.isEnabled = false
    } else if let urlString, urlString.isEmpty {
      navigationItem.rightBarButtonItem?.isEnabled = false
    } else {
      navigationItem.rightBarButtonItem?.isEnabled = true
    }
  }
}

extension ConnectToFunkwhaleViewController: UITableViewDelegate {
  func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {

    tableView.deselectRow(at: indexPath, animated: true)

    guard let cell = tableView.cellForRow(at: indexPath) as? ConnectToFunkwhaleTableViewCell else { return }

    cell.textField.becomeFirstResponder()
  }
}
