//
//  WelcomeView.swift
//  pinniped-ios
//
//  Created by Nathan Mattes on 08.08.23.
//

import UIKit

class WelcomeView: UIView {

  let descriptionLabel: UILabel

  let connectFunkwhaleLabel: UILabel
  let connectFunkwhaleButton: UIButton
  private let connectFunkwhaleButtonWrapper: UIView

  let subscriptionLabel: UILabel
  let subscriptionButton: UIButton
  private let subscriptionButtonWrapper: UIView

  private let contentStackView: UIStackView
  private let contentScrollView: UIScrollView

  override init(frame: CGRect) {

    descriptionLabel = UILabel()
    descriptionLabel.numberOfLines = 0
    descriptionLabel.font = UIFont.preferredFont(forTextStyle: .body)
    descriptionLabel.text = L10n.Welcome.description

    connectFunkwhaleLabel = UILabel()
    connectFunkwhaleLabel.numberOfLines = 0
    connectFunkwhaleLabel.font = UIFont.preferredFont(forTextStyle: .title1)
    connectFunkwhaleLabel.adjustsFontForContentSizeCategory = true
    connectFunkwhaleLabel.text = L10n.Welcome.Steps.ConnectServer.headline

    var connectFunkwhaleButtonConfiguration = UIButton.Configuration.filled()
    connectFunkwhaleButtonConfiguration.title = L10n.Welcome.Steps.ConnectServer.action
    connectFunkwhaleButtonConfiguration.baseBackgroundColor = .tintColor
    connectFunkwhaleButtonConfiguration.baseForegroundColor = .white
    connectFunkwhaleButton = UIButton(configuration: connectFunkwhaleButtonConfiguration)
    connectFunkwhaleButton.translatesAutoresizingMaskIntoConstraints = false

    connectFunkwhaleButtonWrapper = UIView()
    connectFunkwhaleButtonWrapper.addSubview(connectFunkwhaleButton)

    subscriptionLabel = UILabel()
    subscriptionLabel.numberOfLines = 0
    subscriptionLabel.font = UIFont.preferredFont(forTextStyle: .title1)
    subscriptionLabel.adjustsFontForContentSizeCategory = true
    subscriptionLabel.text = L10n.Welcome.Steps.Subscribe.headline

    var subscriptionButtonConfiguration = UIButton.Configuration.filled()
    subscriptionButtonConfiguration.title = L10n.Welcome.Steps.Subscribe.action
    subscriptionButtonConfiguration.baseBackgroundColor = .tintColor
    subscriptionButtonConfiguration.baseForegroundColor = .white
    subscriptionButton = UIButton(configuration: subscriptionButtonConfiguration)
    subscriptionButton.translatesAutoresizingMaskIntoConstraints = false

    subscriptionButtonWrapper = UIView()
    subscriptionButtonWrapper.addSubview(subscriptionButton)

    contentStackView = UIStackView(arrangedSubviews: [descriptionLabel, connectFunkwhaleLabel, connectFunkwhaleButtonWrapper, subscriptionLabel, subscriptionButtonWrapper])
    contentStackView.translatesAutoresizingMaskIntoConstraints = false
    contentStackView.axis = .vertical
    contentStackView.spacing = 16

    contentScrollView = UIScrollView()
    contentScrollView.translatesAutoresizingMaskIntoConstraints = false
    contentScrollView.addSubview(contentStackView)

    super.init(frame: frame)

    addSubview(contentScrollView)

    setupConstraints()
  }

  required init?(coder: NSCoder) { fatalError("init(coder:) has not been implemented") }

  private func setupConstraints() {
    let constraints = [
      contentScrollView.topAnchor.constraint(equalTo: topAnchor),
      contentScrollView.leadingAnchor.constraint(equalTo: leadingAnchor),
      trailingAnchor.constraint(equalTo: contentScrollView.trailingAnchor),
      bottomAnchor.constraint(equalTo: contentScrollView.bottomAnchor),

      contentStackView.topAnchor.constraint(equalTo: contentScrollView.topAnchor),
      contentStackView.leadingAnchor.constraint(equalTo: contentScrollView.leadingAnchor, constant: 16),
      contentScrollView.trailingAnchor.constraint(equalTo: contentStackView.trailingAnchor, constant: 16),
      contentScrollView.bottomAnchor.constraint(equalTo: contentStackView.bottomAnchor),

      contentStackView.widthAnchor.constraint(equalTo: contentScrollView.widthAnchor, constant: -32),

      connectFunkwhaleButton.topAnchor.constraint(equalTo: connectFunkwhaleButtonWrapper.topAnchor),
      connectFunkwhaleButton.centerXAnchor.constraint(equalTo: connectFunkwhaleButtonWrapper.centerXAnchor),
      connectFunkwhaleButtonWrapper.bottomAnchor.constraint(equalTo: connectFunkwhaleButton.bottomAnchor),
      connectFunkwhaleButton.widthAnchor.constraint(greaterThanOrEqualToConstant: 100),

      subscriptionButton.topAnchor.constraint(equalTo: subscriptionButtonWrapper.topAnchor),
      subscriptionButton.centerXAnchor.constraint(equalTo: subscriptionButtonWrapper.centerXAnchor),
      subscriptionButtonWrapper.bottomAnchor.constraint(equalTo: subscriptionButton.bottomAnchor),
      subscriptionButton.widthAnchor.constraint(greaterThanOrEqualToConstant: 100),

    ]

    NSLayoutConstraint.activate(constraints)
  }

  func loginSuccessful() {
    connectFunkwhaleButton.configuration?.baseBackgroundColor = .systemGreen
    connectFunkwhaleButton.configuration?.title = "Connected!"
    connectFunkwhaleButton.isEnabled = false
  }
}
